<?php

class IEX16_Profile {
	const MAX_UNIT_NAME_SIZE = 16;
	const MAX_CHAN_NAME_SIZE = 16;
	const NUM_INPUTS = 16;
	const NUM_CHAN_BANKS = 2;

	private $profile_data;
	private $profile_path;

	public function __construct($filename)
	{
		if(file_exists($filename))
		{
			$this->profile_data = $this->parse($filename);
		}
		else
		{
			$this->profile_data = array();
			$this->profile_data['unit_name'] = "DEFAULT";
			$this->profile_data['channels'] = array();
			for($i = 0; $i < $this::NUM_INPUTS; $i++)
			{
				$this->profile_data['channels'][$i]['input_mode'] = 1;
				$this->profile_data['channels'][$i]['gain'] = 0;
				$this->profile_data['channels'][$i]['name'] = "CHANNEL " . ($i + 1);
			}
			$this->profile_data['adat'] = array(FALSE, FALSE);
			$this->profile_data['str_mask'] = 0xff;
		}
		$this->profile_path = $filename;
	}

	public function get_unit_name()
	{
		return $this->profile_data['unit_name'];
	}

	public function set_unit_name($name)
	{
		$this->profile_data['unit_name'] = $name;
	}

	public function get_str_mask()
	{
		return $this->profile_data['str_mask'];
	}

	public function set_str_mask($mask)
	{
		$this->profile_data['str_mask'] = $mask;
	}

	public function get_channel_input_mode($channel_index)
	{
		switch($this->profile_data['channels'][$channel_index]['input_mode'])
		{
			/*case 0:
				return "M";
				break;*/
			case 1:
				return "DM";
				break;
			case 2:
				return "S";
				break;
			case 3:
				return "N";
				break;
		}
	}

	public function set_channel_input_mode($channel_index, $mode)
	{
		switch($mode)
		{
			/*case "M":
				$this->profile_data['channels'][$channel_index]['input_mode'] = 0;
				break;*/
			case "DM":
				$this->profile_data['channels'][$channel_index]['input_mode'] = 1;
				break;
			case "S":
				$this->profile_data['channels'][$channel_index]['input_mode'] = 2;
				break;
			case "N":
				$this->profile_data['channels'][$channel_index]['input_mode'] = 3;
				break;
		}
	}

	public function get_channel_gain($channel_index)
	{
		switch($this->profile_data['channels'][$channel_index]['gain'])
		{
			case 0x00:
				return "0dB";
				break;
			case 0x06:
				return "6dB";
				break;
			case 0x0c:
				return "12dB";
				break;
		}
	}

	public function set_channel_gain($channel_index, $gain)
	{
		switch($gain)
		{
			case "0dB":
				$this->profile_data['channels'][$channel_index]['gain'] = 0x00;
				break;
			case "6dB":
				$this->profile_data['channels'][$channel_index]['gain'] = 0x06;
				break;
			case "12dB":
				$this->profile_data['channels'][$channel_index]['gain'] = 0x0c;
				break;
		}
	}

	public function get_channel_name($channel_index)
	{
		return $this->profile_data['channels'][$channel_index]['name'];
	}

	public function set_channel_name($channel_index, $name)
	{
		$this->profile_data['channels'][$channel_index]['name'] = $name;
	}

	public function get_locked()
	{
		return $this->profile_data['locked'];
	}

	public function set_locked($locked)
	{
		$this->profile_data['locked'] = $locked;
	}

	public function get_adat($channel_bank)
	{
		if($this->profile_data['adat'][$channel_bank])
		{
			return "adat";
		}
		else
		{
			return "line";
		}
	}

	public function set_adat($channel_bank, $adat)
	{
		if($adat == "adat")
		{
			$this->profile_data['adat'][$channel_bank] = TRUE;
		}
		else
		{
			$this->profile_data['adat'][$channel_bank] = FALSE;
		}
	}

	/* Returns an associative array with the following entries:
	unit_name (string),
	str_mask (integer)
	channels (array of size NUM_INPUTS)
		Each item in channels contains:
		input_mode (integer)
		gain (integer)
		name (string)
	locked (boolean)
	adat (boolean array of size NUM_CHAN_BANKS)
	*/
	private function parse($filename)
	{
		// Initialize arrays so that any code processing output from this
		// function doesn't have to check if the arrays exist.
		$retval = array();
		$retval['channels'] = array_pad(array(), $this::NUM_INPUTS, array());
		$retval['adat'] = array_pad(array(), $this::NUM_CHAN_BANKS, array());

		// Open the file on read binary mode
		$file = fopen($filename, 'rb');
		if($file === FALSE)
		{
			$error = error_get_last();
			throw new Exception("Failed to open file: " . $error['message']);
		}

		// Read in unit name and strMask
		$temp = fread($file, 
			$this::MAX_UNIT_NAME_SIZE // unit_name
			+ 1 // str_mask
			+ 3 // alignment bytes
		);
		$retval = unpack(
			"a" . $this::MAX_UNIT_NAME_SIZE . "unit_name/" . // null-padded string unit_name
			"Cstr_mask" // unsigned byte str_mask
		, $temp);

		// Read in all the channels
		for($i = 0; $i < $this::NUM_INPUTS; $i++)
		{
			$this_channel = array();

			$temp = fread($file, 
				4 + // input_mode
				1 + // gain
				$this::MAX_CHAN_NAME_SIZE + // name
				3 // alignment bytes
			);
			$this_channel = unpack(
				"Ninput_mode/" . // big-endian 32 bit unsigned int input_mode
				"Cgain/" . // unsigned byte gain
				"a" . $this::MAX_CHAN_NAME_SIZE . "name" // null-padded string name
			, $temp);

			$retval['channels'][$i] = $this_channel;
		}

		$temp = fread($file, 1);
		$temp = unpack(
			"Clocked", // unsigned byte locked
			$temp
		);
		if($temp['locked'] == 0)
		{
			$temp = FALSE;
		}
		else
		{
			$temp = TRUE;
		}
		$retval['locked'] = $temp;

		for($i = 0; $i < $this::NUM_CHAN_BANKS; $i++)
		{
			$temp = fread($file, 1);
			$temp = unpack(
				"Cadat", // unsigned byte adat
				$temp
			);
			if($temp['adat'] == 0)
			{
				$temp = FALSE;
			}
			else
			{
				$temp = TRUE;
			}
			$retval['adat'][$i] = $temp;
		}

		fread($file, 1); // alignment byte

		fclose($file);

		return $retval;
	}

	public function save()
	{
		$path = dirname($this->profile_path);
		if(!is_dir($path))
		{
			mkdir($path, 0777, TRUE);
		}
		$this->write($this->profile_path, $this->profile_data);
	}

	// Writes a profile out to a file. See parse() comment for profile format.
	private function write($filename, $profile)
	{
		$file = fopen($filename, 'wb');
		if($file === FALSE)
		{
			$error = error_get_last();
			throw new Exception("Failed to open file: " . $error['message']);
		}

		$temp = pack(
			"a" . $this::MAX_UNIT_NAME_SIZE . // null-padded string
			"C", // unsigned byte
			$profile['unit_name'], 
			$profile['str_mask']
		);
		fwrite($file, $temp);
		fwrite($file, "\0\0\0", 3); // alignment bytes

		// Write out all the channels
		for($i = 0; $i < $this::NUM_INPUTS; $i++)
		{
			$temp = pack(
				"N" . // big endian 32 bit unsigned integer
				"C" . // unsigned byte
				"a" . $this::MAX_CHAN_NAME_SIZE, // null-padded string
				$profile['channels'][$i]['input_mode'], 
				$profile['channels'][$i]['gain'], 
				$profile['channels'][$i]['name']
			);
			fwrite($file, $temp);
			fwrite($file, "\0\0\0", 3); // alignment bytes
		}

		$temp = pack(
			"C", // unsigned byte
			$profile['locked']
		);
		fwrite($file, $temp);
		
		for($i = 0; $i < $this::NUM_CHAN_BANKS; $i++)
		{
			$temp = pack(
				"C", // unsigned byte
				$profile['adat'][$i]
			);
			fwrite($file, $temp);
		}

		fwrite($file, "\0", 1); // alignment byte

		fclose($file);
	}

	// Helper function for other pref list functions
	private static function pref_list_helper($folder)
	{
		$retval = array();

		$prefs = array();
		if(is_dir($folder))
		{
			exec("ls " . $folder, $prefs);
		}

		foreach($prefs as $this_pref)
		{
			$retval[$this_pref] = $this_pref;
		}

		return $retval;
	}

	public static function get_local_saved_prefs()
	{
		require_once("helpers/config.php");

		$path = Config::get_item('saved_iex16_prefs_dir');
		return self::pref_list_helper($path);
	}

	public static function get_external_saved_prefs()
	{
		require_once("helpers/config.php");

		$path = Config::get_item('external_iex16_prefs_dir');
		return self::pref_list_helper($path);
	}

	public static function delete_local_saved_pref($pref_name)
	{
		$path = Config::get_item('saved_iex16_prefs_dir') . $pref_name;
		unlink($path);
	}
}

/* End of iex16_profile.php */
