<?php

class Login {

	private static $token_name = "token";

	// Returns a string of $length random characters
	private static function randomString($length)
	{
		$retval = "";

		for($i = 0; $i < $length; $i++)
		{
			// any ascii character that can be expected to print reasonably
			$temp = mt_rand(32, 126);
			$retval .= chr($temp);
		}

		return $retval;
	}

	private static function get_control_session()
	{
		require_once("helpers/config.php");

		if(file_exists(Config::get_item('control_session_file')))
		{
			$retval['touch_time'] = filemtime(Config::get_item('control_session_file'));
			$contents = file_get_contents(Config::get_item('control_session_file'));
			$contents = explode("\n", $contents);
			$retval['token'] = $contents[0];
			$retval['ip_address'] = $contents[1];
			$retval['username'] = $contents[2];
		}
		else
		{
			$retval['token'] = "";
			$retval['ip_address'] = "";
			$retval['username'] = "";
			$retval['touch_time'] = "";
		}

		return $retval;
	}

	private static function set_control_session($session)
	{
		require_once("helpers/config.php");

		$contents = $session['token'] . "\n" . 
			$session['ip_address'] . "\n" . 
			$session['username'];

		file_put_contents(Config::get_item('control_session_file'), $contents);
	}

	// Returns true if there is a valid control session
	public static function is_logged_in()
	{
		if(session_id() == "")
		{
			session_start();
		}

		$session_data = self::get_control_session();
		if(!empty($_SESSION[self::$token_name]) && 
			!empty($session_data['token']) &&
			!empty($session_data['ip_address']) &&
			$_SESSION[self::$token_name] == $session_data['token'] &&
			$_SERVER['REMOTE_ADDR'] == $session_data['ip_address'] &&
			$session_data['touch_time'] >= (time() - (Config::get_item('control_session_timeout')))
		)
		{
			return TRUE;
		}
		return FALSE;
	}

	// Returns the username and IP address of the user currently logged in.
	// Returns an empty string if no one is logged in.
	public static function get_user_logged_in()
	{
		if(session_id() == "")
		{
			session_start();
		}
		
		$session_data = self::get_control_session();
		if(!empty($session_data['token']) &&
			!empty($session_data['ip_address']) &&
			$session_data['touch_time'] >= (time() - (Config::get_item('control_session_timeout')))
		)
		{
			return $session_data['username'] . " (" . $session_data['ip_address'] . ")";
		}
		else
		{
			return "";
		}
	}

	public static function is_admin_logged_in()
	{
		require_once("models/users.php");

		if(!self::is_logged_in())
		{
			return FALSE;
		}

		$session_data = self::get_control_session();
		if(Users::is_admin($session_data['username']))
		{
			return TRUE;
		}
		return FALSE;
	}

	public static function log_in($username, $password)
	{
		require_once("models/users.php");

		// Admin users can kick standard users or other admin users.
		// Standard users can kick other standard users, but not admin users.
		$current_session = self::get_control_session();
		if(!empty($current_session['username']))
		{
			if(!Users::is_admin($username) && Users::is_admin($current_session['username']))
			{
				return "admin";
			}
		}

		if(Users::authenticate($username, $password))
		{
			if(session_id() == "")
			{
				session_start();
			}

			// Generate user session token
			$generated_token = self::randomString(20);

			// Save user session token
			$session_data['token'] = $generated_token;
			$session_data['ip_address'] = $_SERVER['REMOTE_ADDR'];
			$session_data['username'] = $username;
			self::set_control_session($session_data);

			// Add user session token to the php (html?) session
			$_SESSION[self::$token_name] = $generated_token;
			return "ok";
		}
		else
		{
			return "failed";
		}
	}

	// Redirect to the login page if not authenticated.
	public static function require_login()
	{
		require_once("helpers/url.php");
		require_once("helpers/config.php");
		if(self::is_logged_in())
		{
			touch(Config::get_item('control_session_file'));
		}
		else
		{
			Url::redirect(Config::get_item('url_segment_base') . "login.php");
		}
	}

	// Redirect to the login page if not authenticated or current user is not an admin.
	public static function require_admin_login()
	{
		require_once("helpers/url.php");
		require_once("helpers/config.php");
		if(self::is_admin_logged_in())
		{
			touch(Config::get_item('control_session_file'));
		}
		else
		{
			Url::redirect(Config::get_item('url_segment_base') . "login.php");
		}
	}

	// Remove login session data
	public static function log_out()
	{
		if(file_exists(Config::get_item('control_session_file')))
		{
			unlink(Config::get_item('control_session_file'));
		}
	}
}

/* End of login.php */
